(ns poalses.jonathan.dialect.dialect-nlp
  (:require [dk.simongray.datalinguist :as dl]))


(def nlp
  (dl/->pipeline {:annotators [
                               "tokenize"
                               "ner"]}))

;; Word sets that will show a sentence as being of that dialect

(def australian-words #{"G'day" "aussie" "bogan" "didgeridoo" "bludger" "yobbo" "yobbos" "dinkum" "yakka" "nork" "bogans"})

(def scottish-words #{"laddie" "fook" "fookin" "fooken" "haggis" "daft" "lad" "wee" "fankle" "numpty" "lassies" "lassie" "lass" "laddies" "loch" "kilt"})

(def american-words #{"rad" "lame" "heckin" "slay" "awesome" "heck" "loser" "super" "yay"})

(defn australian-word-predicate
  [sentence]
  (some australian-words (dl/text (dl/tokens sentence))))

(defn scottish-word-predicate
  [sentence]
  (some scottish-words (dl/text (dl/tokens sentence))))

(defn american-word-predicate
  [sentence]
  (some american-words (dl/text (dl/tokens sentence))))

;; Predicate vectors to check a sentence and see if it grammatically matches a dialect

(def australian-predicates
  [australian-word-predicate])

(def scottish-predicates
  [scottish-word-predicate])

(def american-predicates
  [american-word-predicate])


;; Take a sentence and figure out its dialect

(defn detect-sentence-dialect [sentence]
  (let [dialects1  (when ((apply some-fn australian-predicates) sentence) :australian)
        dialects2  (when ((apply some-fn scottish-predicates) sentence) :scottish)
        dialects3  (when ((apply some-fn american-predicates) sentence) :american)
        dialects   (remove nil? [dialects1 dialects2 dialects3])]
    (if (empty? dialects)
      [:standard]
      dialects)))

(cond)

(defn detect-sample-dialect [sample]
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp sample)))))))))

(defn detect-dialects
  [filename]
  (let [filedata (clojure.edn/read-string (slurp filename))
        key-results (map detect-sample-dialect (keys filedata))
        expected-results (map keyword (vals filedata))]
    (let [result-comparison (map zero? (map compare key-results expected-results))]
      (println (str (count (filter true? result-comparison)) "/" (count result-comparison) " correct.")))))

